#!/bin/sh


# replaces the echo calls
echo_dbg() {
  # dump trace info if we're running from wuninst
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      echo $@
  fi
}

# ==================================================================
# Initialize variables : list of objects, resources to clean up
# ===================================================================

init_variables()
{
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
    set -x
  fi
  set +e
  
  # This runs as a FW task on a managed node.
  # As a task, there will be my node name in $ENDPOINT, and my node oid in $ENDPOINT_OID
  # If these environment vars don't indicate ManagedNode,
  # then possibly we're experiencing the FW wuninst bug where it prefers to run
  # the TMR uninstallation on an LCF endpoint that has the same label as a managed node.

  if [ -z "$ENDPOINT" ] ; then
    echo_dbg "Problem: expected ENDPOINT environment variable."
    exit 2
  fi

  if [ -z "$ENDPOINT_OID" ] ; then
    echo_dbg "Problem: expected ENDPOINT_OID environment variable."
    exit 2
  fi


  TST=`echo "$ENDPOINT_OID" | sed -n /ManagedNode/p`
  RC=$?
  if [ $RC -ne 0 ]; then
    # probably blew up because of no sed on lcf endpoint
    echo_dbg "Expected to run on a managed node."
    exit 2
  fi

  if [ x"$TST" = x ] ; then
    # ManagedNode not found in $ENDPOINT_OID
    echo_dbg "Expected to run on a managed node."
    exit 2
  fi




  OSERV=`objcall 0.0.0 get_oserv 2> /dev/null`
  RC=$?
  if [ $RC -ne 0 ]; then
    echo_dbg "Problem: objcall 0.0.0 get_oserv not functioning. Uninstall exiting."
    exit 2
  fi
  export OSERV


  # By now, pretty sure of things working managed node style
  # but if they don't then exit if one of the following commands
  # fails.
  set -e



  # more information along the lines of $ENDPOINT and $ENDPOINT_OID
  # but from a different source

  BASEOID=`objcall 0.0.0 self`
  HOSTOID=`objcall ${BASEOID} getattr HostLocation`
  QUOTED_HOSTNAME=`idlcall $HOSTOID SysAdmin::PolicyDrivenBase::_get_label`
  export QUOTED_HOSTNAME





  # get database dir
  DBDIR=`objcall $OSERV query db_dir 2>/dev/null`
  export DBDIR




  RN=`wtmrname`
  CLO=`wlookup "Library#$RN"`
  TNR=`wlookup "NameRegistry#$RN"`
  PCO=`wlookup -r Classes "Presentation#$RN"`
  PRCO=`wlookup -r Classes "PolicyRegion#$RN"`
  PMCO=`wlookup -r Classes "ProfileManager#$RN"`


  IRO=`wlookup InterRegion`
  export IRO


  eval IRONAME=`idlattr -t -g $IRO name string`
  export IRONAME

  MGTREGIONNAME="Messaging and Collaboration"
  MGTREGION_IRONAME="${MGTREGIONNAME}#${IRONAME}"
  export MGTREGIONNAME
  export MGTREGION_IRONAME



  # am I running on the TMR server?
  TMRSERVEROID=`wlookup ServerManagedNode`

  if [ "$ENDPOINT_OID" = "$TMRSERVEROID" ] ; then
    ISTMRSERVER="1"
  else
    ISTMRSERVER="0"
  fi


  INTERP=`objcall $OSERV query interp 2> /dev/null`
  export INTERP


  # get interp-specific things
  INST_DIR=`objcall $OSERV query install_dir`
  case "$INTERP" in
    os2-ix86|w32-ix86)
      # PC interps have back slashes; convert to forward slashes
      INST_DIR=`echo $INST_DIR | sed 's-\\\\-/-g'`
      RMTMDUTILITY=rmtmd.exe
      ;;

    *)
      # leave install dir as is on unix
      RMTMDUTILITY=rmtmd
      ;;
  esac
  export INST_DIR


  TMP=`wtemp`
  export TMP


  # create BINDIR setting
  BINDIR=$INST_DIR/$INTERP
  export BINDIR

  NOTESDIR="$INST_DIR/generic_unix/TME/DominoManager"
  export NOTESDIR


  # this is where rmtmd(.exe) lives, my C-code utility for uninstall
  if [ $WUNINST -eq 1 ] ; then
    NOTESBINDIR="$BINDIR/TME/DominoManager"
  else
    NOTESBINDIR="."
  fi
  export NOTESBINDIR



  # this is where rmtmd(.exe) lives, my C-code utility for uninstall
  if [ $WUNINST -eq 1 ] ; then
    LOTUSDOMDIR="$BINDIR/TME/DominoManager/LotusDominoServer"
  else 
    LOTUSDOMDIR="./$INTERP"
  fi
  export LOTUSDOMDIR



  if test -f "$LOTUSDOMDIR/$RMTMDUTILITY" ; then
    # nop
    NOP=1
  else
    echo_dbg "Problem: expected to find $LOTUSDOMDIR/$RMTMDUTILITY program."
    exit 2
  fi







  # Keep track of the overall success.
  # If OVERALL is non-zero at the end of the run means something went wrong.
  OVERALL=0
  export OVERALL








}






remove_productinfo()
{
  # For every one ProductInfo object, there's a related list of PatchInfo objects
  # whose name, when mangled up a little, is the name of the patch tag files
  # placed in the .installed directories. Use the PatchInfo objects to delete
  # the tag files.
  #
  # parm 1 product        ie DominoPAC_51 or DominoManager_3.1
  # parm 2 node           ie "trumpet"

  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      set -x
  fi

  set +e


  PRODTAG=$1
  PROD="$1#$IRONAME"
  NODE="$2"
  STATUS=0


  PROD_IO=`wlookup -r ProductInfo "$PROD" 2>/dev/null || wlookup -r PatchInfo "$PROD" 2>/dev/null`
  if [ $? -eq 0 ] ; then


    PATCHINFOS=`idlcall $PROD_IO _get_patches`
    NUMPATCHES=`idlarg 1 $PATCHINFOS`
    NUMPATCHES=`expr "$NUMPATCHES" + 2 2>/dev/null`

    # Call remove_host_locations on our PatchInfo objects and
    # remove references to this host.
    COUNT=2
    while [ "$COUNT" -lt "$NUMPATCHES" ]
    do
      PATCHOID=`idlarg $COUNT $PATCHINFOS`
      idlcall $PATCHOID remove_host_locations '{ 1' $NODE '}'
      RC=$?
      echo_dbg "remove_host_locations from PatchInfo rc $RC"
      STATUS=`expr "$STATUS" + $RC`

      # next
      COUNT=`expr "$COUNT" + 1 `
    done


    # Infer the names of the patch tag files and remove them, or
    # actually parse and inspect the output of get_host_locations (same parms
    # and remove_host_locations) and rm -f with additional precision.
    COUNT=2
    while [ "$COUNT" -lt "$NUMPATCHES" ]
    do
      PATCHOID=`idlarg $COUNT $PATCHINFOS`
      PATCHLABEL=`idlcall $PATCHOID _get_label`
      RC=$?
      eval PATCHLABEL=$PATCHLABEL

       if [ $RC -eq 0 ]; then
       if [ x"$PATCHLABEL" = x ]; then
         STATUS=`expr "$STATUS" + 1`
       else
          echo_dbg "Deleting patch tag files for patch $PATCHLABEL"
          rm -f "$DBDIR/.installed/pa_${PATCHLABEL}_ALIDB"
          rm -f "$BINDIR/../generic_unix/.installed/pa_${PATCHLABEL}_GBIN"
          rm -f "$BINDIR/../generic/.installed/pa_${PATCHLABEL}_JBIN"
          rm -f "$BINDIR/../lcf_bundle/.installed/pa_${PATCHLABEL}_LCF"
          rm -f "$BINDIR/../lcf_bundle.40/.installed/pa_${PATCHLABEL}_LCFNEW"
          rm -f "$BINDIR/.installed/pa_${PATCHLABEL}_BIN"
          rm -f "$BINDIR/../../msg_cat/.installed/pa_${PATCHLABEL}_CAT"
          rm -f "$BINDIR/../../lib/$INTERP/.installed/pa_${PATCHLABEL}_LIB"
          rm -f "$BINDIR/contrib/.installed/pa_${PATCHLABEL}_CONTRIB"

          # some patch tags don't being with pa_ !!!
          rm -f "$DBDIR/.installed/${PATCHLABEL}_ALIDB"
          rm -f "$BINDIR/../generic_unix/.installed/${PATCHLABEL}_GBIN"
          rm -f "$BINDIR/../generic/.installed/${PATCHLABEL}_JBIN"
          rm -f "$BINDIR/../lcf_bundle/.installed/${PATCHLABEL}_LCF"
          rm -f "$BINDIR/../lcf_bundle.40/.installed/${PATCHLABEL}_LCFNEW"
          rm -f "$BINDIR/.installed/${PATCHLABEL}_BIN"
          rm -f "$BINDIR/../../msg_cat/.installed/${PATCHLABEL}_CAT"
          rm -f "$BINDIR/../../lib/$INTERP/.installed/${PATCHLABEL}_LIB"
          rm -f "$BINDIR/contrib/.installed/${PATCHLABEL}_CONTRIB"

       fi
      else
       STATUS=`expr "$STATUS" + 1`
      fi
      COUNT=`expr "$COUNT" + 1 `
    done



    # remove the product tag files
    echo_dbg "Deleting product tag files for product $PRODTAG"
    rm -f $DBDIR/.installed/${PRODTAG}_ALIDB
    rm -f $DBDIR/.installed/pa_${PRODTAG}_ALIDB
    rm -f $BINDIR/../generic_unix/.installed/${PRODTAG}_GBIN
    rm -f $BINDIR/../generic_unix/.installed/pa_${PRODTAG}_GBIN
    rm -f $BINDIR/../generic/.installed/${PRODTAG}_JBIN
    rm -f $BINDIR/../generic/.installed/pa_${PRODTAG}_JBIN
    rm -f $BINDIR/../lcf_bundle/.installed/${PRODTAG}_LCF
    rm -f $BINDIR/../lcf_bundle/.installed/pa_${PRODTAG}_LCF
    rm -f $BINDIR/../lcf_bundle.40/.installed/${PRODTAG}_LCFNEW
    rm -f $BINDIR/../lcf_bundle.40/.installed/pa_${PRODTAG}_LCFNEW
    rm -f $BINDIR/.installed/${PRODTAG}_BIN
    rm -f $BINDIR/.installed/pa_${PRODTAG}_BIN
    rm -f $BINDIR/../../msg_cat/.installed/${PRODTAG}_CAT
    rm -f $BINDIR/../../msg_cat/.installed/pa_${PRODTAG}_CAT
    rm -f $BINDIR/../../lib/$INTERP/.installed/${PRODTAG}_LIB
    rm -f $BINDIR/../../lib/$INTERP/.installed/pa_${PRODTAG}_LIB
    rm -f $BINDIR/contrib/.installed/${PRODTAG}_CONTRIB
    rm -f $BINDIR/contrib/.installed/pa_${PRODTAG}_CONTRIB

    # Remove add'l files that are not removed from notescontents
    rm -f $LIBDIR/libDominoPerformanceStation.a
    rm -f $LIBDIR/libDominoPerformanceStation.dll
    rm -f $LIBDIR/libLotusDominoServer.a
    rm -f $LIBDIR/libLotusDominoServer.dll
    rm -f $DBDIR/TMDRAS_*
    rm -f $BINDIR/bin/LotusDominoServer.pl
    rm -f $BINDIR/../../msg_cat/C/AboutDominoManager_3.1Catalog.cat
    rm -f $BINDIR/../lcf_bundle*/msg_cat/C/Domino30HelpCat.cat
    rm -f $BINDIR/../lcf_bundle*/msg_cat/C/Domino30MonCat.cat
    rm -f $BINDIR/../lcf_bundle*/msg_cat/C/DominoPerformanceStationMsg.cat


    # remove this host from the ProductInfo object
    idlcall $PROD_IO remove_host_locations '{ 1' $NODE '}'
    RC=$?
    echo_dbg "remove_host_locations from ProductInfo object rc $RC"
    STATUS=`expr "$STATUS" + $RC`



    OVERALL=`expr "$OVERALL" + "$STATUS"`


    if [ "$OVERALL" -eq 0 ]; then

      if [ "$ISTMRSERVER" = "1" ] ; then

        # remove patch objects from the TMR
        COUNT=2
        while [ "$COUNT" -lt "$NUMPATCHES" ]
        do
          PATCHOID=`idlarg $COUNT $PATCHINFOS`
          idlcall $PATCHOID remove
          COUNT=`expr "$COUNT" + 1 `
        done

        # all hosts should be gone from the ProductInfo object
        PROD_LOC=`idlcall ${PROD_IO} _get_locations`
        PROD_REMAINING=`echo ${PROD_LOC} | sed 's/}.*//g' | cut -d\{ -f2`
        if [ $PROD_REMAINING = 0 ]; then
          x=`wdel "@ProductInfo:${PROD}" 2>&1 || wdel "@PatchInfo:${PROD}" 2>&1`
          RC=$?
          echo_dbg "$x"
          echo_dbg "wdel @ProductInfo:$PROD rc $RC"
          OVERALL=`expr "$OVERALL" + $RC`
        else
          echo_dbg "Unexpected hosts ($PROD_LOC) returned by _get_locations. ProductInfo object $PROD not deleted."
          OVERALL=`expr "$OVERALL" + 1`
        fi

      fi
    else
      echo_dbg "Problem encountered in remove_productinfo"
    fi

  else
    echo_dbg "ProductInfo object $PROD already removed."
  fi
}















remove_collection()
{
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      set -x
  fi
  
  echo_dbg "Removing Sentry Collection $1 ..."
  COL=`wlookup -r MonitoringCapabilityCollection "$1#$RN"`
  if [ $? -eq 0 ] ; then
      remove_object $COL MonitoringCapabilityCollection
  fi
}






remove_object()
{
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      set -x
  fi
  OID=$1
  TEXT=$2
  echo_dbg "Deleting $TEXT Object gently with idlcall. OID= $OID"
  idlcall -T top $OID remove > /dev/null 2>&1
  RC=$?
  echo_dbg "idlcall -T top $OID remove rc $RC"

  if [ $RC -ne 0 ] ; then
      echo_dbg "Deleting $TEXT Object forcefully with objcall $OID _delete"
      objcall $OID _delete
      RC=$?
      echo_dbg "objcall $OID _delete rc $RC"
  fi

  OVERALL=`expr $OVERALL + $RC`
}















# do this on the TMR server

ServerMain()
{
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      set -x
  fi
  set +e


  # Remove instances of LDS, DPS, Events4Profiles
  echo "Removing application objects ..."

  x=`$LOTUSDOMDIR/rmtmd DeleteObjects LotusDominoServer 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects LotusDominoServer rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  x=`$LOTUSDOMDIR/rmtmd DeleteObjects DominoPerformanceStation 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects DominoPerformanceStation rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  x=`$LOTUSDOMDIR/rmtmd DeleteObjects Events4Profile 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects Events4Profile rc $RC"
  OVERALL=`expr $OVERALL + $RC`




  if [ $OVERALL -eq 0 ]; then

    # Remove dependency sets. Check for existence before deleting.
    # Some may be, some may not be, depending on the version and patches
    # installed.
    echo "Removing dependency sets ..."

    DEPSETS="DominoPerformanceStation_dep \
             DominoPerformanceStation_dep1 \
             DominoPerformanceStation_dep2 \
             LotusDominoServerBase_dep \
             LotusDominoServerBase_dep1 \
             LotusDominoServerBase_dep2 \
             LotusDominoServerBase_dep3 \
             LotusDominoServerBase_dep4 \
             LotusDominoServerBase_dep5 \
             LotusDominoServerPD_dep"

    set +e

    for DEP in $DEPSETS
    do
      x=`wlookup -r DependencyMgr "$DEP#$RN" 2>&1`
      RC=$?
      echo_dbg "$x"
      if [ $RC -eq 0 ] ; then
        echo_dbg "Removing depset $DEP"
        x=`wdepset -d "@DependencyMgr:$DEP#$RN" 2>&1`
        echo_dbg "$x"
        RC2=$?
        echo_dbg "wdepset rc deleting $DEP $RC2"
        OVERALL=`expr $OVERALL + $RC2`
      fi
    done
  fi


  # defect 147750: not removing jobs in the TMD Configuration Task Library
  set +e
  x=`wdeljob "Configure Notes Client Install 3.1"  "Client Distribution Operations 3.1" 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "wdeljob Configure Notes Client Install 3.1 rc $RC"
  OVERALL=`expr $OVERALL + $RC`
  
  x=`wdeljob "Configure Notes Client Upgrade 3.1"  "Client Distribution Operations 3.1" 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "wdeljob Configure Notes Client Upgrade 3.1 rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  # Delete task and task libraries of ours, tmd3 and pac5.
  echo "Removing task libraries ..."
  x=`$LOTUSDOMDIR/rmtmd DeleteTasks 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteTasks  rc $RC"
  OVERALL=`expr $OVERALL + $RC`







  set +e
  x=`wlookup -r ProductInfo "DominoPAC_51" 2>&1 || wlookup -r PatchInfo "DominoPAC_51" 2>&1`
  RC=$?
  echo_dbg "$x"
  if [ $RC -eq 0 ] ; then

    # remove resource models according to notescontents
    x=`$LOTUSDOMDIR/rmtmd RemoveResourceModels 2>&1`
    RC=$?
    echo_dbg "$x"
    echo_dbg "rmtmd RemoveResourceModels rc $RC"
    OVERALL=`expr $OVERALL + $RC`

    # deletes the resource model profiles we installed
    x=`$LOTUSDOMDIR/rmtmd DeleteProfiles Tmw2kProfile 2>&1`
    RC=$?
    echo_dbg "$x"
    echo_dbg "rmtmd DeleteProfiles Tmw2kProfile rc $RC"
    OVERALL=`expr $OVERALL + $RC`

  fi


  set +e
  x=`wlookup -r ProductInfo "DominoManager_3.1" 2>&1`
  RC=$?
  echo_dbg "$x"
  if [ $RC -eq 0 ] ; then

    # deletes classic DM profiles installed in the 3.1 time frame
    x=`$LOTUSDOMDIR/rmtmd DeleteProfiles SentryProfile 2>&1`
    RC=$?
    echo_dbg "$x"
    echo_dbg "rmtmd DeleteProfiles SentryProfile rc $RC"
    OVERALL=`expr $OVERALL + $RC`

    if [ $RC -eq 0 ]; then
      # Remove monitor collections if left over from TMD3-to-PAC5 upgrade
      echo "Removing monitor collections ..."
      remove_collection  "Domino Performance Station Collection"
      remove_collection  "Domino v4.6 Real Time Collection"
      remove_collection  "Domino v4.6 Statistics Collection"
      remove_collection  "Domino v5.0 Real Time Collection"
      remove_collection  "Domino v5.0 Statistics Collection"
      remove_collection  "Domino v6.0 Real Time Collection"
      remove_collection  "Domino v6.0 Statistics Collection"
    fi
  fi




  # DeleteProfileManagers we installed
  echo "Removing profile managers ..."
  x=`$LOTUSDOMDIR/rmtmd DeleteProfileManagers 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteProfileManagers rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  # Will remove PR, but only if it's empty
  echo "Removing policy regions ..."
  x=`$LOTUSDOMDIR/rmtmd RemovePolicyRegion "Manager For Domino 3.1" 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd RemovePolicyRegion Manager For Domino 3.1  rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  # Will remove PR, but only if it's empty
  echo_dbg "Removing PolicyRegion Messaging and Collaboration"
  x=`$LOTUSDOMDIR/rmtmd RemovePolicyRegion "Messaging and Collaboration" 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd RemovePolicyRegion Messaging and Collaboration rc $RC"

  # Not a stopper if it can't be totally removed because Exchange may reside in there, too.
  # OVERALL=`expr $OVERALL + $RC`


  # Will remove PR, but only if it's empty
  echo_dbg "Removing PolicyRegion DominoDefaultsRegion"
  x=`$LOTUSDOMDIR/rmtmd RemovePolicyRegion "DominoDefaultsRegion" 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd RemovePolicyRegion DominoDefaultsRegion rc $RC"
  OVERALL=`expr $OVERALL + $RC`






  # Remove notice group from TMD3 if left over
  NOTICE_GROUP="Domino Manager 3.1"
  echo "Removing notice groups ..."
  NG=`wlookup -r TMF_Notice "$NOTICE_GROUP#$RN" 2>/dev/null`
  if [ $? -eq 0 ] ; then
     x=`idlcall $NG remove 2>&1`
     RC=$?
     echo_dbg "$x"
     echo_dbg "idcall $NG remove rc $RC"
     OVERALL=`expr $OVERALL + $RC`
  fi



  # Remove notice group from PAC5 if left over
  NOTICE_GROUP="Domino Messaging and Collaboration"
  echo_dbg "Removing Notice Group $NOTICE_GROUP"
  NG=`wlookup -r TMF_Notice "$NOTICE_GROUP#$RN" 2>/dev/null`
  if [ $? -eq 0 ] ; then
     idlcall $NG remove
     RC=$?
     echo_dbg "idcall $NG remove rc $RC"
     OVERALL=`expr $OVERALL + $RC`
  fi









  # If, by this point, there were any problems, then DO NOT remove our class objects
  # because we need them to perform remove operations.

  if [ $OVERALL -ne 0 ]; then
    # Go no further.
    echo "Not all objects were removed. Remove unwanted items from the $MGTREGION region and re-run uninstall."
    if [ $WUNINST -eq 1 ] ; then
        echo "See $TMP/wuninst.log."
    fi
    exit 1
  fi




  # Remove our classes as Managed Resource from all policy regions.
  set +e

  MANAGED_RESOURCES="LotusDominoServer DominoPerformanceStation Events4Profile"

  echo "Removing managed resources ..."
  for CLASS in $MANAGED_RESOURCES
  do
    CO=`wlookup -r Classes "$CLASS#$RN" 2>/dev/null`
    if [ $? -eq 0 ] ; then
      echo_dbg "Removing resource $CLASS from Policy Regions"
      regions=`objcall $PRCO _get_members`
      for j in $regions
      do
        # remove_instance_manager returns 0 if present, 1 if not present
        idlcall -T top $j TMF_Policy::Region::remove_instance_manager $CO 1>/dev/null 2>&1
      done
    fi
  done







  # Remove the run-time classes.




  ALL_CLASSES="LotusDominoServer LotusDominoServerPD LotusDominoServerPV LotusDominoServerUpcall \
  Events4Profile DominoPerformanceStation DominoPerformanceStationPD DominoPerformanceStationPV \
  imp_DominoManagerRas::TMDRas TMDRasUpcall"

  echo "Removing classes ..."


  for CLASS in $ALL_CLASSES
  do

      CO=`idlcall $CLO TMF_SysAdmin::Library::lookup_object ''\"$CLASS\"' {0}'`
      if [ $? -ne 0 ] ; then
          continue
      fi

      echo_dbg "Removing class $CLASS"


      PROTOLIST=`idlattr -t -g $CO prototypes TMF_Types::ObjectList`


      NUM_PROTOS=`idlarg 1 $PROTOLIST`
      i=1
      while [ $i -le $NUM_PROTOS ]
      do
        i=`expr $i + 1`
        PROTO_OID=`idlarg $i $PROTOLIST`

        echo_dbg "Removing prototype item OID=$PROTO_OID"

        # this call returns 1 even if successful
        idlcall -T top $PROTO_OID remove > /dev/null 2>&1

      done

      #
      # Remove the class object.
      #

      echo_dbg "Removing class object $CO"

      # this call can return 1  even if successful
      idlcall -T top $CO remove > /dev/null 2>&1

  done









  # Remove distinguished objects.

  DISTINGUISHED_OBJECTS="TMDRas"

  echo "Removing distinguished objects ..."

  for OBJECT in $DISTINGUISHED_OBJECTS
  do
     CO=`wlookup "$OBJECT#$RN" 2>/dev/null`
     if [ $? -eq 0  ] ; then
         echo_dbg "Removing distinguished object $OBJECT: $CO"

         # this call can return 1  even if successful
         x=`idlcall -T top $CO remove 2>&1`
         echo_dbg "$x"
     fi
  done








  # Clean out the name registry.
  #

  for CLASS in $ALL_CLASSES
  do
     echo_dbg "Unregistering class $CLASS from TNR"
     wregister -u -r Classes $CLASS > /dev/null 2>&1
     if [ $? -eq 0 ] ; then
         idlcall -T top $TNR TMF_TNR::Resource::remove ''\"$CLASS\"' TRUE' > /dev/null 2>&1
         RC=$?
         echo_dbg "idlcall -T top $TNR TMF_TMR::Resource::remove $CLASS TRUE rc $RC"
     fi
  done










  for OBJECT in $DISTINGUISHED_OBJECTS
  do
     echo_dbg "Unregistering distinguished object $OBJECT from TNR"
     wregister -u -r distinguished $OBJECT > /dev/null 2>&1
     if [ $? -ne 0 ] ; then
         idlcall -T top $TNR TMF_TNR::Resource::remove ''\"$OBJECT\"' TRUE' > /dev/null 2>&1
         RC=$?
         echo_dbg "idlcall -T top $TNR TMF_TMR::Resource::remove $OBJECT TRUE rc $RC"
     fi
  done








  #
  # Remove any Presentation objects
  #
  echo "Removing presentation objects ..."

  PRESENTATIONS="LotusDominoServer DominoPerformanceStation"


  for i in $PRESENTATIONS
  do
     PO=`objcall $PCO _lookup_object $i`
     if [ $? -eq 0 ] ; then
         echo_dbg "Removing Presentation object for $i"
         remove_object $PO Presentation
     fi
  done




  # Remove roles from every admin and the tmr, too
  echo "Removing roles ..."
  x=`$LOTUSDOMDIR/rmtmd RemoveRoles Domino_Admin Domino_DPS_Admin Domino_DPS_User Domino_Monitor Domino_TDP_Admin Domino_TDP_User Domino_User 2>&1`
  RC=$?
  echo_dbg "$x"
  OVERALL=`expr $OVERALL + $RC`










  if [ $OVERALL -gt 0 ]; then
    echo "Problem detected removing class objects. Correct and re-run uninstall."
    if [ $WUNINST -eq 1 ] ; then
        echo "See $TMP/wuninst.log."
    fi
  else

    # Most of everything is removed. Do these final things. There's no going back.

    # Remove product info object and patchinfo objects and patch tag id files
    echo "Removing product info objects and tag files ..."
    remove_productinfo DominoManager_3.1 "${QUOTED_HOSTNAME}"
    remove_productinfo "DominoPAC_51" "${QUOTED_HOSTNAME}"
    remove_productinfo "DominoPAC_511" "${QUOTED_HOSTNAME}"

    echo_dbg "Unregistering DominoPAC_51 Uninstall"
    PRODUCTINFO="DominoPAC_51"
    x=`wregister -u -r Uninstall "$PRODUCTINFO" 2>&1`
    echo_dbg "$x"

    echo_dbg "Unregistering DominoManager_3.1 Uninstall"
    PRODUCTINFO="DominoManager_3.1"
    x=`wregister -u -r Uninstall "$PRODUCTINFO" 2>&1`
    echo_dbg "$x"

  fi
}









#-----------------------------------------------------------------------------------------
# just uninstall a node


NodeMain()
{
  if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
      set -x
  fi
  set +e

  # Clean up TMD3 from this MN/GW
  x=`$LOTUSDOMDIR/rmtmd DeleteObjects LotusDominoServer $ENDPOINT_OID 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects LotusDominoServer rc $RC"
  OVERALL=`expr $OVERALL + $RC`

  x=`$LOTUSDOMDIR/rmtmd DeleteObjects DominoPerformanceStation $ENDPOINT_OID 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects DominoPerformanceStation  rc $RC"
  OVERALL=`expr $OVERALL + $RC`

  x=`$LOTUSDOMDIR/rmtmd DeleteObjects Events4Profile $ENDPOINT_OID 2>&1`
  RC=$?
  echo_dbg "$x"
  echo_dbg "rmtmd DeleteObjects Events4Profile rc $RC"
  OVERALL=`expr $OVERALL + $RC`


  # this subroutine will just delete product and patch tag files for this node
  # because we're not on the TMR server

  echo "Removing product info objects and tag files ..."
  remove_productinfo DominoManager_3.1 "${QUOTED_HOSTNAME}"
  remove_productinfo DominoPAC_51 "${QUOTED_HOSTNAME}"
  remove_productinfo DominoPAC_511 "${QUOTED_HOSTNAME}"
}











###############################################################################
#
# MAIN
#

# set a flag to determine if we're running through wuninst (1)
if [ -n "$ENDPOINT" ] ; then
  WUNINST=1
else
  WUNINST=0
fi
export WUNINST

# look for the -debug argument
DEBUG=0
for i in $@; do
  if [ $i = "-debug" -o $i = "-DEBUG" ]; then
    DEBUG=1
  fi
done

# enable tracing if we're running from wuninst
if [ $WUNINST -eq 1 -o $DEBUG -eq 1 ] ; then
    set -x
fi

PRODUCT="Tivoli Manager for Domino/ITM for Messaging and Collaboration"

if [ $WUNINST -eq 1 ] ; then
    USAGE="wuninst {DominoManager_3.1 | DominoPAC_51 | DominoPAC_511} Host [-rmfiles]

where Host is the label of the Managed Node from which the product will be 
removed.  Select this node name from the list of produced by this command: 

    wlookup -ar ManagedNode

If you want to remove the product from a single Managed Node, provide that node
name instead of the TMR node name.
"
else
    USAGE="uninstall.sh [-rmfiles]

Please note that this script needs to be run on each Managed Node from which
you would like the product uninstalled.
"
fi

# look for the -usage argument passed by wuninst
for i in $@; do
  if [ $i = "-usage" -o $i = "-USAGE" ]; then
  echo "

Uninstall for $PRODUCT

This utility will remove Tivoli Manager for Domino (TMD) 3.1 (tag =
DominoManager_3.1) and IBM Tivoli Monitoring for Messaging and Collaboration 
(ITMMC): Lotus Domino (tag = DominoPAC_51 or DominoPAC_511).  Most servers will
have one or the other, but a server upgraded from TMD 3.1 to ITMMC 5.1 will 
have both products installed.  Specifying one tag or the other causes both 
products to be removed.

    $USAGE 

Before running this utility, delete user-defined profiles, profile managers,
jobs, and policy regions.  This utility will only delete objects that were
created during installation and all object instances of LotusDominoServer, 
DominoPerformanceStation, and Events4Profile.

If uninstall cannot remove the $PRODUCT policy region because
there are user-defined items remaining, uninstall will exit.  Please manually 
delete the remaining items, and run this utility again.

"

    exit 0
  fi
done

# check for Tivoli environment
if [ $WUNINST -ne 1 -a -z "$BINDIR" ] ; then
  echo "Please run this from a Tivoli command prompt"
  exit 1
fi


# look for the -rmfiles argument passed by wuninst
RMFILES=0
for i in $@; do
  if [ $i = "-rmfiles" -o $i = "-RMFILES" ]; then
    RMFILES=1
  fi
done
export RMFILES


# if we're running outside of wuninst, we need to setup these vars
if [ $WUNINST -ne 1 ] ; then
    BASEOID=`objcall 0.0.0 self`
    ENDPOINT_OID=`objcall ${BASEOID} getattr HostLocation`
    ENDPOINT=`idlcall $ENDPOINT_OID SysAdmin::PolicyDrivenBase::_get_label`
fi

# set all the other vars
init_variables

# prompt the user to continue to be safe
if [ $WUNINST -ne 1 ] ; then
  echo -n "Proceed with uninstall? [n] "
  read answer
  if [ "$answer" = "Y" -o "$answer" = "y" ] ; then
      echo Uninstalling ...
  else 
      echo Exiting.
      exit 0
  fi
fi

if [ "$ISTMRSERVER" = "1" ] ; then
  # running on TMR server, so delete the whole lot
  ServerMain
else
  # running on a node in the TMR
  NodeMain
fi


set +e



# if there's a problem, stop.

if [ $OVERALL -ne 0 ]; then
  # Go no further.
  echo "Not all objects were removed. Remove unwanted items from the Domino region(s) and re-run uninstall."
  exit 1
fi




# lastly delete the files since there's no errors so far

if [ "$RMFILES" = "1" ] ; then


  echo "Removing files ..."

  # routine in C to delete the files here
  x=`$LOTUSDOMDIR/rmtmd DeleteFiles 2>&1`
  echo_dbg "$x"




  # Although the DeleteFiles subfn removes directories,
  # remove the notesbindir one more time because I know it didn't get removed on NT
  # while rmtmd.exe was loaded and running from that directory.
  if [ $WUNINST -eq 1 ] ; then
    rm -fr $NOTESBINDIR
  fi


fi

echo "Uninstall is complete."

exit $OVERALL

